## $Id: document.py 10842 2013-12-11 13:21:37Z henrik $
##
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
These are the document tickets.
"""
import grok
from datetime import datetime
from grok import index
from hurry.workflow.interfaces import IWorkflowInfo, IWorkflowState
from zope.event import notify
from zope.component import getUtility
from zope.component.interfaces import IFactory
from zope.interface import implementedBy
from zope.i18n import translate
from waeup.ikoba.interfaces import IIkobaUtils
from waeup.ikoba.interfaces import MessageFactory as _
from waeup.ikoba.documents.interfaces import IDocument, IDocument
from waeup.ikoba.utils.helpers import attrs_to_fields, get_current_principal
from waeup.ikoba.utils.logger import Logger

class Document(grok.Container, Logger):
    """This is a document.
    """
    grok.implements(IDocument)
    grok.provides(IDocument)
    grok.baseclass()

    logger_name = 'waeup.ikoba.${sitename}.documents'
    logger_filename = 'documents.log'
    logger_format_str = '"%(asctime)s","%(user)s",%(message)s'

    def logger_info(self, comment=None):
        """Get the logger's info method.
        """
        self.logger.info('%s' % comment)
        return

    def __init__(self):
        super(Document, self).__init__()
        self.id = None
        return

Document = attrs_to_fields(Document)

class DocumentFactory(grok.GlobalUtility):
    """A factory for documents.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.Document')
    title = u"Create a new document.",
    description = u"This factory instantiates new documents."

    def __call__(self, *args, **kw):
        return Document(*args, **kw)

    def getInterfaces(self):
        return implementedBy(Document)

@grok.subscribe(IDocument, grok.IObjectAddedEvent)
def handle_document_added(document, event):
    """If a document is added the transition create is fired.
    The latter produces a logging message.
    """
    if IWorkflowState(document).getState() is None:
        IWorkflowInfo(document).fireTransition('create')
    return
